/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.expression.model.types.TDouble;
import org.fortiss.af3.platform.language.executable.ExecutableBase;
import org.fortiss.af3.platform.raspberry.generator.executable.framework.IWriteableExecutable;
import org.fortiss.af3.platform.raspberry.model.ConsoleOutput;
import org.fortiss.af3.project.model.typesystem.IType;

/** Executable for {@link ConsoleOutput}. */
public class ConsoleOutputExecutable extends ExecutableBase<ConsoleOutput> implements
		IWriteableExecutable {
	/** Constructor. */
	public ConsoleOutputExecutable(ConsoleOutput modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getWriteCode(String prefix, String singletonPrefix, OutputPort logicalSignal,
			String value) {
		String typeFormat = getTypeFormat(logicalSignal.getVariableType());
		String name = logicalSignal.getName();
		return "printf(\"" + name + " = " + typeFormat + "\\n\", " + value + ");\n";
	}

	/** Returns the C format string for the given type. */
	private String getTypeFormat(IType type) {
		if(type instanceof TDouble) {
			return "%f";
		}
		return "%i";
	}

	/** {@inheritDoc} */
	@Override
	public String
			getNoValWriteCode(String prefix, String singletonPrefix, OutputPort logicalSignal) {
		return "printf(\"" + logicalSignal.getName() + " = NoVal\\n\");\n";
	}
}
