/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.platform.raspberry.generator.executable.framework.ISingletonInitializationExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.framework.ISingletonTerminationExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.framework.IWriteableExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.library.PiHALLibraryExecutableBase;
import org.fortiss.af3.platform.raspberry.model.ActuatorPWM;

/** Executable for {@link ActuatorPWM}. */
public class PWMActuatorExecutable extends PiHALLibraryExecutableBase<ActuatorPWM> implements
		ISingletonInitializationExecutable, IWriteableExecutable, ISingletonTerminationExecutable {
	/** Constructor. */
	public PWMActuatorExecutable(ActuatorPWM modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public String getHeaderFileName() {
		return "temp_actuator.h";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonIdentifier() {
		// Using header file as unique ID
		return getHeaderFileName();
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonVariableDeclarationCode(String singletonPrefix) {
		return "static int " + singletonPrefix + "_actuator_pwm = -1;\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonInitializationCode(String singletonPrefix) {
		return singletonPrefix + "_actuator_pwm = temp_actuator_initialize(\"/dev/ttyACM0\");\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonTerminationCode(String singletonPrefix) {
		return "temp_actuator_terminate();\n";
	}

	/** {@inheritDoc} */
	@Override
	public String getWriteCode(String prefix, String singletonPrefix, OutputPort logicalSignal,
			String value) {
		return "temp_actuator_device_set_target(" + singletonPrefix + "_actuator_pwm, " +
				modelElement.getChannelID() + ", " + value + ");\n";
	}

	/** {@inheritDoc} */
	@Override
	public String
			getNoValWriteCode(String prefix, String singletonPrefix, OutputPort logicalSignal) {
		return null;
	}
}
